/*
*   Copyright (C) 2003 Crissi <crissi99@gmx.de>
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*/

#include "kpum.h"
#include "useradmin.h"

#include <qlabel.h>
#include <qpixmap.h>
#include <kmainwindow.h>
#include <klocale.h>
#include <kpopupmenu.h>
#include <kaboutkde.h>
#include <kaboutapplication.h>
#include <qmessagebox.h>
#include <qlayout.h>
#include <kstatusbar.h>
#include <kaboutdata.h>
#include <kaboutapplication.h>
#include <kaboutkde.h>
#include <kedittoolbar.h>
#include <kstdaction.h>
#include <klocale.h>

KPum::KPum() : KMainWindow( 0, "KPum" ) {

	initStatusbar();

	// set the shell's ui resource file
	setXMLFile( "kpumui.rc" );

	setIcon( QPixmap( "kpum.png" ) );

	mainMenu = new KMenuBar( this );
	KPopupMenu *filemenu = new KPopupMenu( this );

	menuBar() ->insertItem( i18n( "&File" ), filemenu );

	filemenu->insertItem( i18n( "&Quit" ), this, SLOT( close() ), CTRL + Key_Q );
	mainMenu->insertItem( i18n( "&Help" ), helpMenu( QString::null, false ) );
	user_admin = new UserAdmin( this );
	user_admin->show();
	user_admin->setFocus();
	initAction();
	setCentralWidget( user_admin );
		
	user_admin->config->setGroup( "General Options" );
		WindowSize = QSize( user_admin->config->readNumEntry( "Mainwindow width", 50 ), user_admin->config->readNumEntry( "Mainwindow height", 50 ) );
	WindowPos = QPoint (user_admin->config->readNumEntry( "Mainwindow position x", 10 ), user_admin->config->readNumEntry( "Mainwindow position y", 10 ) );

	if ( WindowSize.width() < 50 || WindowSize.height() < 50 ) {
		WindowSize = QSize( minimumWidth(), minimumHeight() );
	}

	if ( WindowPos.x() < 0 || WindowPos.y() < 0 )
		WindowPos = QPoint ( 0, 0 );

	resize( WindowSize );
	move ( WindowPos );
	
	//   this->showMaximized();
	//  userAdmin->showMaximized();
	statusBar() ->message( i18n( "Ready" ), 2000 );
}

KPum::~KPum() {}

void KPum::initAction() {
	//( void* ) KStdAction::quit( this, SLOT( quitCalled() ), actionCollection() );

	saveAction = new KAction( i18n( "&Save" ), "fileexport", Key_S, user_admin, SLOT( saveConfig() ), actionCollection(), "save" );
	deleteUserAction = new KAction( i18n( "&Delete user..." ), "editdelete", Key_D, user_admin, SLOT( slotDelUser() ), actionCollection(), "deleteUser" );
	newUserAction = new KAction( i18n( "&New User..." ), "newfont", Key_N, user_admin, SLOT( slotAddUser() ), actionCollection(), "newUser" );

	KStdAction::preferences( user_admin, SLOT( slotShowPref() ), actionCollection(), "preferences" );
	KStdAction::helpContents( this, SLOT( helpClicked() ), actionCollection(), "help" );
	KStdAction::quit( this, SLOT( quitClicked() ), actionCollection(), "quit" );

	showToolbarAction = KStdAction::showToolbar( this, SLOT( slotShowToolbar() ), actionCollection() );
	showStatusbarAction = KStdAction::showStatusbar( this, SLOT( slotShowStatusbar() ), actionCollection() );

	( void* ) KStdAction::keyBindings( this, SLOT( slotKeyBindings() ), actionCollection() );
	( void* ) KStdAction::configureToolbars( this, SLOT( slotConfigToolbars() ), actionCollection() );
	( void* ) KStdAction::configureNotifications( this, SLOT( slotConfigNotifications() ), actionCollection() );

	createGUI();

}
void KPum::initStatusbar() {
	statusBar() ->insertItem( i18n( "Ready." ), 1, 100 );
	statusBar() ->setItemAlignment( 1, AlignLeft | AlignVCenter );
}

void KPum::slotShowToolbar() {
	if ( !showToolbarAction->isChecked() ) {
		toolBar( "mainToolBar" ) ->hide();
	} else {
		toolBar( "mainToolBar" ) ->show();
	}
}
void KPum::slotShowStatusbar() {
	if ( !showStatusbarAction->isChecked() ) {
		statusBar() ->hide();
	} else {
		statusBar() ->show();
	}
}
void KPum::slotKeyBindings() {
	//	KKeyDialog::configureKeys( actionCollection(), xmlFile() );
}
void KPum::slotConfigToolbars() {
	KEditToolbar dlg( actionCollection() );

	if ( dlg.exec() ) {
		createGUI();
	}
}

void KPum::helpClicked() {
	// show help
}

void KPum::slotConfigNotifications() {
}

void KPum::quitClicked() {
	user_admin->config->setGroup( "General Options" );
	user_admin->config->writeEntry( "Mainwindow width", width() );
	user_admin->config->writeEntry( "Mainwindow height", height() );
	user_admin->config->writeEntry( "Mainwindow position x", x() );
	user_admin->config->writeEntry( "Mainwindow position y", y() );
	user_admin->config->sync();
	close();
}

#include "kpum.moc"
